package com.example.witsystem_device_plugin;

import android.bluetooth.BluetoothDevice;
import android.content.Context;
import android.os.Handler;
import android.util.Log;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.alibaba.fastjson.JSON;
import com.example.witsystem_device_plugin.SDK.channel.ChannelName;
import com.example.witsystem_device_plugin.SDK.event.AddDeviceHandler;
import com.example.witsystem_device_plugin.SDK.event.BleConnectStateHandler;
import com.example.witsystem_device_plugin.SDK.event.DeviceChangeHandler;
import com.example.witsystem_device_plugin.SDK.vibrate.Vibrate;
import com.witsystem.top.witsystem_android_ble_sdk.WitsystemSDK;
import com.witsystem.top.witsystem_android_ble_sdk.WitsystmBleSDK;
import com.witsystem.top.witsystem_android_ble_sdk.ble.call.AddDeviceScanCallBack;
import com.witsystem.top.witsystem_android_ble_sdk.device.call.AddDeviceCallBack;
import com.witsystem.top.witsystem_android_ble_sdk.device.manage.DeviceManage;
import com.witsystem.top.witsystem_android_ble_sdk.device.call.BleCallBack;
import com.witsystem.top.witsystem_android_ble_sdk.device.call.BleProgressCommandCall;
import com.witsystem.top.witsystem_android_ble_sdk.device.call.BleSendCommandCall;
import com.witsystem.top.witsystem_android_ble_sdk.device.call.BleUnlockCallBack;
import com.witsystem.top.witsystem_android_ble_sdk.device.call.OtaCall;
import com.witsystem.top.witsystem_android_ble_sdk.device.call.OtaUpgradeCall;
import com.witsystem.top.witsystem_android_ble_sdk.device.code.DeviceFunctionType;
import com.witsystem.top.witsystem_android_ble_sdk.device.command.CardType;
import com.witsystem.top.witsystem_android_ble_sdk.device.command.CommandMessage;
import com.witsystem.top.witsystem_android_ble_sdk.device.command.DeviceModel;
import com.witsystem.top.witsystem_android_ble_sdk.device.command.FaceType;
import com.witsystem.top.witsystem_android_ble_sdk.device.command.MarkType;
import com.witsystem.top.witsystem_android_ble_sdk.device.command.PasswordType;
import com.witsystem.top.witsystem_android_ble_sdk.device.entity.DeviceSoftwareInfo;
import com.witsystem.top.witsystem_android_ble_sdk.device.entity.OtaEntityResult;
import com.witsystem.top.witsystem_android_ble_sdk.device.entity.StateCode;
import com.witsystem.top.witsystem_android_ble_sdk.error.InitErrorCode;
import com.witsystem.top.witsystem_android_ble_sdk.error.NotInitialized;
import com.witsystem.top.witsystem_android_ble_sdk.util.ByteToString;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import SDK.err.Error;
import SDK.model.Devices;
import SDK.net.NetResponse;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

/**
 * WitsystemDevicePlugin
 */
public class WitsystemDevicePlugin implements FlutterPlugin, MethodCallHandler {

    private static final String TAG = "插件SDK";

    private MethodChannel channel;

    private Context applicationContext;

    private WitsystemSDK witsystmBleSDK;

    private final AddDeviceHandler addDeviceHandler = new AddDeviceHandler();

    private final BleConnectStateHandler bleConnectStateHandler = new BleConnectStateHandler();


    private final DeviceChangeHandler deviceChangeHandler = new DeviceChangeHandler();

    private final ExecutorService executorService = Executors.newFixedThreadPool(2);

    private final Handler handler = new Handler();

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        applicationContext = flutterPluginBinding.getApplicationContext();
        channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), ChannelName.witsystemSDK);
        channel.setMethodCallHandler(this);

        EventChannel scanningChannel = new EventChannel(flutterPluginBinding.getBinaryMessenger(), ChannelName.multipleAddDevice);
        scanningChannel.setStreamHandler(addDeviceHandler);

        EventChannel bleConnectStateChannel = new EventChannel(flutterPluginBinding.getBinaryMessenger(), ChannelName.bleConnectState);
        bleConnectStateChannel.setStreamHandler(bleConnectStateHandler);

        EventChannel deviceChangeChannel = new EventChannel(flutterPluginBinding.getBinaryMessenger(), ChannelName.deviceChange);
        deviceChangeChannel.setStreamHandler(deviceChangeHandler);
    }


    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        executorService.execute(() -> {
            if (!call.method.equals("initSDK") && (witsystmBleSDK == null || !witsystmBleSDK.isInit())) {
                result.error(String.valueOf(InitErrorCode.NOT_INITIALIZED.getCode()), InitErrorCode.NOT_INITIALIZED.getError(), null);
                return;
            }
            try {
                switch (call.method) {
                    case "initSDK":
                        initSDK(call.argument("appId"), call.argument("token"), call.argument("appSecret"), call.argument("rsaPrivateKey"), result);
                        return;
                    case "uninitDevices"://取消初始化
                        witsystmBleSDK.uninitialized();
                        break;
                    case "bleUnlock":
                        bleUnlock(call.argument("deviceId"), result);
                        return;
                    case "longUnlock":
                        longUnlock(call.argument("deviceId"), result);
                        return;
                    case "connectAndOpen":
                        connectAndOpen(call.argument("deviceId"), call.argument("isOpen"), result);
                        return;
                    case "scanAddBleDevice":
                        startAddScan();
                        break;
                    case "stopAddBleDevice":
                        stopAddBleDevice();
                        break;
                    case "addBleDevice":
                        addBleDevice(call.argument("deviceId"), call.argument("deviceName"), result);
                        return;
                    case "closeAddDevice":
                        closeAddDevice(call.argument("deviceId"), result);
                        return;
                    case "addPassword":
                        addPassword(call.argument("deviceId"), call.argument("passwordType"), call.argument("password"), call.argument("passwordName"), call.argument("userId"), result);
                        return;
                    case "deletePassword":
                        deletePassword(call.argument("deviceId"), call.argument("uid"), result);
                        return;
                    case "addCard":
                        addCard(call.argument("deviceId"), call.argument("cardType"), call.argument("cardName"), call.argument("userId"), result);
                        return;
                    case "cancelAddCard":
                        cancelAddCard(call.argument("deviceId"));
                        break;
                    case "deleteCard":
                        deleteCard(call.argument("deviceId"), call.argument("uid"), result);
                        return;
                    case "freezeCard":
                        freezeCard(call.argument("deviceId"), call.argument("uid"), call.argument("isFreeze"), result);
                        return;
                    case "closeOrOpenCard":
                        closeOrOpenCard(call.argument("deviceId"), call.argument("isClose"), result);
                        return;
                    case "addMark":
                        addMark(call.argument("deviceId"), call.argument("markType"), call.argument("markName"), call.argument("userId"), result);
                        return;
                    case "cancelAddMark":
                        cancelAddMark(call.argument("deviceId"));
                        break;
                    case "deleteMark":
                        deleteMark(call.argument("deviceId"), call.argument("uid"), result);
                        return;
                    case "freezeMark":
                        freezeMark(call.argument("deviceId"), call.argument("uid"), call.argument("isFreeze"), result);
                        return;
                    case "addFace":
                        addFace(call.argument("deviceId"), call.argument("faceType"), call.argument("faceName"), call.argument("userId"), result);
                        break;
                    case "cancelAddFace":
                        cancelAddFace(call.argument("deviceId"));
                        break;
                    case "deleteFace":
                        deleteFace(call.argument("deviceId"), call.argument("uid"), result);
                        return;
                    case "freezeFace":
                        freezeFace(call.argument("deviceId"), call.argument("uid"), call.argument("isFreeze"), result);
                        return;
                    case "updateVolume"://修改设备的音量
                        updateVolume(call.argument("deviceId"), call.argument("volume"), result);
                        return;
                    case "setModel"://设置设备的模式
                        witsystmBleSDK.getBleDevice().setModel(call.argument("deviceId"), Enum.valueOf(DeviceModel.class, call.argument("model")), call(call.argument("deviceId"), result));
                        return;
                    case "updateDeviceState":
                        updateDeviceState(call.argument("deviceId"), result);
                        return;
                    case "closeBleDevice":
                        closeBleDevice(call.argument("deviceId"));
                        break;
                    case "checkUpVersion":
                        checkUpVersion(call.argument("deviceId"), result);
                        return;
                    case "startBleOta":
                        startBleOta(call.argument("deviceId"), result);
                        return;
                    case "stopBleOta":
                        stopBleOta(call.argument("deviceId"));
                        break;
                    case "deviceList":
                        deviceList(result);
                        return;
                    case "device":
                        device(call.argument("deviceId"), result);
                        return;
                    case "isDeviceFunction"://查询指定设置是否支持指定的功能
                        boolean deviceFunction = witsystmBleSDK.getDeviceManage().isDeviceFunction(call.argument("deviceId"), Enum.valueOf(DeviceFunctionType.class, call.argument("type")));
                        result.success(deviceFunction);
                        return;
                    case "deviceUserList":
                        deviceUserList(call.argument("deviceId"), result);
                        return;
                    case "freezeOrUnfreeze":
                        freezeOrUnfreeze(call.argument("deviceId"), call.argument("freezeUserUuid"), call.argument("isFreeze"), result);
                        return;
                    case "manageDeleteDeviceUser":
                        manageDeleteDeviceUser(call.argument("deviceId"), call.argument("deleteUserUuid"), result);
                        return;
                    case "sendDevice":
                        sendDevice(call.argument("userId"), call.argument("deviceId"), call.argument("startDate"), call.argument("endDate"), call.argument("authType"), call.argument("repeatType"), result);
                        return;
                    case "translationDevice":
                        translationDevice(call.argument("userId"), call.argument("deviceId"), result);
                        return;
                    case "deleteDevices":
                        deleteDevices(call.argument("deviceId"), result);
                        return;
                    case "lockRecord":
                        lockRecord(call.argument("deviceId"), call.argument("page"), result);
                        return;
                    case "refurbishDevice"://刷新指定设备数据
                        result.success(witsystmBleSDK.getDeviceManage().getDeviceService().refurbishDevice(call.argument("deviceId")));
                        return;
                    case "refurbishAllDevice"://刷新所有设备数据
                        result.success(witsystmBleSDK.getDeviceManage().getDeviceService().refurbishAllDevice());
                        return;
                    case "getDeviceStateInfo"://获得设备的状态信息
                        StateCode stateCode = witsystmBleSDK.getDeviceManage().getDeviceStateInfo(call.argument("deviceId"));
                        result.success(stateCode == null ? null : JSON.toJSONString(stateCode));
                        return;
                    case "getDeviceSoftwareFunction"://获得设备软件的支持信息
                        DeviceSoftwareInfo info = witsystmBleSDK.getDeviceManage().getDeviceSoftwareFunction(call.argument("deviceId"));
                        result.success(info == null ? null : JSON.toJSONString(info));
                        return;
                    case "updateDeviceName"://获得设备软件的支持信息
                        updateDeviceName(call.argument("deviceId"), call.argument("deviceName"), result);
                        return;
                    case "bindRelay"://绑定网关
                        bindRelay(call.argument("superDeviceId"), call.argument("deviceId"), result);
                        return;
                    case "unbindRelay"://解绑绑定网关
                        unbindRelay(call.argument("deviceId"), result);
                        return;
                    case "startVibrate"://开启震动
                        if (call.argument("pattern") == null) {
                            Vibrate.getInstance(applicationContext).start();
                        } else {
                            Vibrate.getInstance(applicationContext).start(call.argument("pattern"), call.argument("repeat"));
                        }
                        break;
                    case "stopVibrate"://关闭震动
                        Vibrate.getInstance(applicationContext).stop();
                        break;
                    default:
                        result.notImplemented();
                        break;
                }

            } catch (NotInitialized notInitialized) {
                notInitialized.printStackTrace();
                result.error(String.valueOf(notInitialized.getCode()), notInitialized.getMessage(), null);
                return;
            }
            result.success("SUCCESS");
        });

    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
    }

    /**
     * 初始化SDK
     *
     * @param appId         app唯一识别码
     * @param token         用户tokem
     * @param appSecret     app密钥
     * @param rsaPrivateKey rsa 私钥
     * @param result        返回回调
     */
    private void initSDK(String appId, String token, String appSecret, String rsaPrivateKey, Result result) {
        witsystmBleSDK = WitsystmBleSDK.getInstance(applicationContext, appId, token, appSecret, rsaPrivateKey);
        witsystmBleSDK.initSDK(new BleCallBack<WitsystmBleSDK>() {
            @Override
            public void success(WitsystmBleSDK witsystmBleSDK) {
                Log.i(TAG, "success: 初始化成功" + witsystmBleSDK);
                try {
                    result.success(true);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void fail(int i, String s) {
                result.error(i + "", s, null);
            }
        });

    }


    /*-------------------------------- 设备的蓝牙操作------------------------------*/

    /**
     * 启动添加扫描
     */
    private void startAddScan() {
        witsystmBleSDK.getAddDevice().scanAddBleDevice(new AddDeviceScanCallBack() {
            @Override
            public void fail(int i, String s) {
                handler.post(() -> addDeviceHandler.onError(i, s));
            }

            @Override
            public void scanDevice(BluetoothDevice bluetoothDevice, int i, byte[] bytes) {
                Map<String, Object> map = new HashMap<>();
                map.put("address", bluetoothDevice.getAddress());
                map.put("deviceId", bluetoothDevice.getName());
                map.put("rssi", i);
                map.put("bytes", ByteToString.bytesToHexString(bytes));
                handler.post(() -> addDeviceHandler.onScanResult(map));
            }
        });

    }


    /**
     * 停止添加扫描
     */
    private void stopAddBleDevice() {
        try {
            witsystmBleSDK.getAddDevice().stopScanAddBleDevice();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    /**
     * 添加设备
     */
    private void addBleDevice(String deviceId, String deviceName, Result result) {
        witsystmBleSDK.getAddDevice().addBleDevice(deviceId, deviceName, new AddDeviceCallBack<Devices>() {
            @Override
            public void waitSettingsState(String s) {
                handler.post(() -> channel.invokeMethod("waitSettingsState", s));
            }

            @Override
            public void success(Devices devices) {
                result.success(JSON.toJSONString(devices));
            }

            @Override
            public void fail(int i, String s) {
                result.error(String.valueOf(i), s, null);
            }
        });
    }

    /**
     * 关闭添加设备
     *
     * @param deviceId 设备ID
     */
    private void closeAddDevice(String deviceId, Result result) {
        witsystmBleSDK.getAddDevice().closeAddDevice(deviceId);
        result.success("SUCCESS");
    }


    /**
     * 开启设备
     */
    private void bleUnlock(String deviceId, Result result) {
        BleUnlockCallBack<Devices> bleUnlockCallBack = new BleUnlockCallBack<Devices>() {
            @Override
            public void success(Devices devices) {
                Map<String, Object> map = new HashMap<>();
                List<Devices> listDevice = new ArrayList<>(1);
                listDevice.add(devices);
                map.put("type", "UNLOCK");
                map.put("devices", listDevice);
                result.success(JSON.toJSONString(map));
            }

            @Override
            public void fail(int i, String s) {
                result.error(String.valueOf(i), s, null);
            }

            @Override
            public void foundMultipleDevices(List<BluetoothDevice> list) {
                Log.i(TAG, "foundMultipleDevices: 扫描到多个设备" + list.size());
                List<Devices> listDevice = new ArrayList<>();
                for (BluetoothDevice bluetoothDevice : list) {
                    listDevice.add(DeviceManage.getInstance(null).device(bluetoothDevice.getName()));
                }
                Map<String, Object> map = new HashMap<>();
                map.put("type", "DEVICES");
                map.put("devices", listDevice);
                result.success(JSON.toJSONString(map));
            }
        };
        if (deviceId == null) {
            witsystmBleSDK.getUnlock().bleUnlock(bleUnlockCallBack);
        } else {
            witsystmBleSDK.getUnlock().bleUnlock(deviceId, bleUnlockCallBack);
        }
    }


    /**
     * 连接设备并且可以开启设备
     */
    private void connectAndOpen(String deviceId, boolean isOpen, Result result) {
        Log.i(TAG, "connectAndOpen: 连接并开门" + witsystmBleSDK.getDeviceManage().device(deviceId));
        witsystmBleSDK.getBleDevice().connectAndOpen(deviceId, isOpen, call(deviceId, result));
    }


    /**
     * 通过蓝牙添加密码
     */
    private void addPassword(String deviceId, String passwordType, String password, String passwordName, String userId, Result result) throws NotInitialized {
        witsystmBleSDK.getBleDevice().addPassword(deviceId, Enum.valueOf(PasswordType.class, passwordType), password, passwordName, userId, call(deviceId, result));
    }


    /**
     * 通过蓝牙删除密码
     */
    private void deletePassword(String deviceId, String uid, Result result) throws NotInitialized {
        witsystmBleSDK.getBleDevice().deletePassword(deviceId, uid, call(deviceId, result));
    }

    /**
     * 通过蓝牙添加卡
     */
    private void addCard(String deviceId, String cardType, String cardName, String userId, Result result) throws NotInitialized {
        witsystmBleSDK.getBleDevice().addCard(deviceId, Enum.valueOf(CardType.class, cardType), cardName, userId, call(deviceId, result));
    }


    /**
     * 取消添加卡
     */
    private void cancelAddCard(String deviceId) throws NotInitialized {
        witsystmBleSDK.getBleDevice().cancelAddCard(deviceId);
    }

    /**
     * 通过蓝牙删除卡
     */
    private void deleteCard(String deviceId, String uid, Result result) throws NotInitialized {
        witsystmBleSDK.getBleDevice().deleteCard(deviceId, uid, call(deviceId, result));
    }


    /**
     * 通过蓝牙冻结卡
     */
    private void freezeCard(String deviceId, String uid, boolean isFreeze, Result result) throws NotInitialized {
        witsystmBleSDK.getBleDevice().freezeCard(deviceId, uid, isFreeze, call(deviceId, result));
    }

    /**
     * 通过蓝牙关闭或者开启卡功能
     */
    private void closeOrOpenCard(String deviceId, boolean isClose, Result result) throws NotInitialized {
        witsystmBleSDK.getBleDevice().closeOrOpenCard(deviceId, isClose, call(deviceId, result));
    }


    /**
     * 通过蓝牙添加指纹
     */
    private void addMark(String deviceId, String markType, String markName, String userId, Result result) throws NotInitialized {
        witsystmBleSDK.getBleDevice().addMark(deviceId, Enum.valueOf(MarkType.class, markType), markName, userId, new BleProgressCommandCall<CommandMessage>() {
            @Override
            public void progress(int sum, int value) {
                Map<String, Integer> map = new HashMap<>();
                map.put("sum", sum);
                map.put("value", value);
                handler.post(() -> channel.invokeMethod("progress", JSON.toJSONString(map)));
            }

            @Override
            public void sendSuccess() {
                handler.post(() -> channel.invokeMethod("sendSuccess", null));
            }

            @Override
            public void commandError(String code, String error) {
                //指令的异常，比如校验不正确
                result.error(code, error, null);
            }

            @Override
            public void fail(int code, String error) {
                result.error(String.valueOf(code), error, null);
            }

            @Override
            public void success(CommandMessage message) {
                result.success(JSON.toJSONString(witsystmBleSDK.getDeviceManage().device(deviceId)));
            }
        });
    }

    /**
     * 取消添加指纹
     */
    private void cancelAddMark(String deviceId) throws NotInitialized {
        witsystmBleSDK.getBleDevice().cancelAddMark(deviceId);
    }


    /**
     * 通过蓝牙删除指纹
     */
    private void deleteMark(String deviceId, String uid, Result result) throws NotInitialized {
        witsystmBleSDK.getBleDevice().deleteMark(deviceId, uid, call(deviceId, result));
    }


    /**
     * 通过蓝牙冻结指纹
     */
    private void freezeMark(String deviceId, String uid, boolean isFreeze, Result result) throws NotInitialized {
        witsystmBleSDK.getBleDevice().freezeMark(deviceId, uid, isFreeze, call(deviceId, result));
    }


    /**
     * 通过蓝牙添加人脸
     */
    private void addFace(String deviceId, String faceType, String markName, String userId, Result result) throws NotInitialized {
        witsystmBleSDK.getBleDevice().addFace(deviceId, Enum.valueOf(FaceType.class, faceType), markName, userId, call(deviceId, result));
    }

    /**
     * 取消添加人脸
     */
    private void cancelAddFace(String deviceId) throws NotInitialized {
        witsystmBleSDK.getBleDevice().cancelAddFace(deviceId);
    }

    /**
     * 通过蓝牙删除人脸
     */
    private void deleteFace(String deviceId, String uid, Result result) throws NotInitialized {
        witsystmBleSDK.getBleDevice().deleteFace(deviceId, uid, call(deviceId, result));
    }

    /**
     * 通过蓝牙冻结人脸
     */
    private void freezeFace(String deviceId, String uid, boolean isFreeze, Result result) throws NotInitialized {
        witsystmBleSDK.getBleDevice().freezeCard(deviceId, uid, isFreeze, call(deviceId, result));
    }

    /**
     * 通过蓝牙修改音量
     */
    private void updateVolume(String deviceId, int volume, Result result) throws NotInitialized {
        witsystmBleSDK.getBleDevice().updateVolume(deviceId, volume, call(deviceId, result));
    }


    /**
     * 通过蓝牙同步设备状态
     */
    private void updateDeviceState(String deviceId, Result result) throws NotInitialized {
        witsystmBleSDK.getBleDevice().updateDeviceState(deviceId, call(deviceId, result));
    }

    /**
     * 通过关闭指定设备的蓝牙操作
     */
    private void closeBleDevice(String deviceId) throws NotInitialized {
        witsystmBleSDK.getBleDevice().closeBleDevice(deviceId);
    }

    /**
     * 通过关闭指定设备的蓝牙操作
     */
    private void checkUpVersion(String deviceId, Result result) throws NotInitialized {
        witsystmBleSDK.getOta().checkUpVersion(deviceId, new OtaCall<OtaEntityResult>() {
            @Override
            public void fail(int i, String s) {
                result.error(String.valueOf(i), s, null);
            }

            @Override
            public void success(OtaEntityResult otaEntityResult) {
                result.success(JSON.toJSON(otaEntityResult));
            }
        });
    }


    /**
     * 开始蓝牙ota
     */
    private void startBleOta(String deviceId, Result result) throws NotInitialized {
        witsystmBleSDK.getOta().startBleOta(deviceId, new OtaUpgradeCall() {
            @Override
            public void commandError(String code, String error) {
                result.error(code, error, null);
            }

            @Override
            public void progress(double v) {
                handler.post(() -> channel.invokeMethod("progress", v));
            }

            @Override
            public void fail(int code, String error) {
                try {
                    result.error(String.valueOf(code), error, null);
                } catch (Exception ignored) {
                }
            }

            @Override
            public void success(String version) {
                result.success(version);
            }
        });
    }

    /**
     * 停止蓝牙升级
     */
    private void stopBleOta(String deviceId) throws NotInitialized {
        witsystmBleSDK.getOta().stopBleOta(deviceId);
    }


    /**
     * 返回统一回调信息
     */
    private BleSendCommandCall<CommandMessage> call(String deviceId, Result result) {
        return new BleSendCommandCall<CommandMessage>() {
            @Override
            public void sendSuccess() {
                handler.post(() -> channel.invokeMethod("sendSuccess", null, new Result() {
                    @Override
                    public void success(@Nullable Object result) {
                        Log.i(TAG, "success");
                    }

                    @Override
                    public void error(String errorCode, @Nullable String errorMessage, @Nullable Object errorDetails) {
                        Log.i(TAG, "error: " + errorMessage);
                    }

                    @Override
                    public void notImplemented() {
                        Log.i(TAG, "notImplemented");
                    }
                }));
            }

            @Override
            public void commandError(String code, String error) {
                try {
                    result.error(code, error, error);//指令的异常，比如校验不正确
                } catch (Exception ignored) {
                }
            }

            @Override
            public void fail(int code, String error) {
                try {
                    result.error(String.valueOf(code), error, error);
                } catch (Exception ignored) {
                }

            }

            @Override
            public void success(CommandMessage message) {
                result.success(JSON.toJSONString(witsystmBleSDK.getDeviceManage().device(deviceId)));
            }
        };
    }


    /*-------------------------------- 设备的网络操作------------------------------*/


    /**
     * 返回设备列表
     */
    private void deviceList(Result result) throws NotInitialized {
        result.success(JSON.toJSONString(witsystmBleSDK.getDeviceManage().devicesList()));
    }

    /**
     * 返回指定设备的信息
     */
    private void device(String deviceId, Result result) throws NotInitialized {
        result.success(JSON.toJSONString(witsystmBleSDK.getDeviceManage().device(deviceId)));
    }

    /**
     * 获取指定设备的用户列表
     */
    private void deviceUserList(String deviceId, Result result) throws NotInitialized {
        NetResponse netResponse = witsystmBleSDK.getDeviceManage().deviceUserList(deviceId);
        if (netResponse.getErr() == Error.SUCCESS.getCode()) {
            result.success(JSON.toJSONString(netResponse));
        } else {
            result.error(String.valueOf(netResponse.getErr()), netResponse.getMsg(), netResponse.getData());
        }

    }


    /**
     * 远程开门
     */
    private void longUnlock(String deviceId, Result result) throws NotInitialized {
        try {
            NetResponse netResponse = witsystmBleSDK.getUnlock().longUnlock(deviceId);
            if (netResponse.getErr() == Error.SUCCESS.getCode()) {
                result.success(JSON.toJSONString(netResponse));
            } else {
                result.error(String.valueOf(netResponse.getErr()), netResponse.getMsg(), netResponse.getData());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    /**
     * 冻结或者解冻用户
     */
    private void freezeOrUnfreeze(String deviceId, String freezeUserUuid, boolean isFreeze, Result result) throws NotInitialized {
        NetResponse netResponse = witsystmBleSDK.getDeviceManage().freezeOrUnfreeze(deviceId, freezeUserUuid, isFreeze);
        if (netResponse.getErr() == Error.SUCCESS.getCode()) {
            result.success(JSON.toJSONString(netResponse));
        } else {
            result.error(String.valueOf(netResponse.getErr()), netResponse.getMsg(), netResponse.getData());
        }
    }

    /**
     * 管理员或者高等级用户删除用户权限
     */
    private void manageDeleteDeviceUser(String deviceId, String deleteUserUuid, Result result) throws NotInitialized {
        NetResponse netResponse = witsystmBleSDK.getDeviceManage().manageDeleteDeviceUser(deviceId, deleteUserUuid);
        if (netResponse.getErr() == Error.SUCCESS.getCode()) {
            result.success(JSON.toJSONString(netResponse));
        } else {
            result.error(String.valueOf(netResponse.getErr()), netResponse.getMsg(), netResponse.getData());
        }

    }

    /**
     * 发送设备
     *
     * @param deviceId   设备ID
     * @param userId     接收者账号
     * @param startDate  开始时间
     * @param endDate    结束时间
     * @param authType   权限类型
     * @param repeatType 重复类型
     */
    private void sendDevice(String userId, String deviceId, String startDate, String endDate, String authType, String repeatType, Result result) throws NotInitialized {
        Map<String, String> map = new HashMap<>();
        map.put("userId", userId);
        map.put("deviceId", deviceId);
        map.put("authType", authType);
        if (startDate != null) map.put("startDate", startDate);
        if (endDate != null) map.put("endDate", endDate);
        if (repeatType != null) map.put("repeatType", repeatType);

        NetResponse netResponse = witsystmBleSDK.getDeviceManage().getDeviceService().sendDevice(map);
        if (netResponse.getErr() == Error.SUCCESS.getCode()) {
            result.success(JSON.toJSONString(netResponse));
        } else {
            result.error(String.valueOf(netResponse.getErr()), netResponse.getMsg(), netResponse.getData());
        }

    }

    /**
     * 转让设备
     *
     * @param deviceId 设备ID
     * @param userId   接收者账号
     */
    private void translationDevice(String userId, String deviceId, Result result) throws NotInitialized {
        NetResponse netResponse = witsystmBleSDK.getDeviceManage().getDeviceService().translationDevice(userId, deviceId);
        if (netResponse.getErr() == Error.SUCCESS.getCode()) {
            result.success(JSON.toJSONString(netResponse));
        } else {
            result.error(String.valueOf(netResponse.getErr()), netResponse.getMsg(), netResponse.getData());
        }

    }

    /**
     * 删除设备
     *
     * @param deviceId 设备ID
     */
    private void deleteDevices(String deviceId, Result result) throws NotInitialized {
        NetResponse netResponse = witsystmBleSDK.getDeviceManage().getDeviceService().deleteDevices(deviceId);
        if (netResponse.getErr() == Error.SUCCESS.getCode()) {
            result.success(JSON.toJSONString(netResponse));
        } else {
            result.error(String.valueOf(netResponse.getErr()), netResponse.getMsg(), netResponse.getData());
        }

    }

    /**
     * 获得开门记录
     *
     * @param deviceId 设备ID
     * @param page     第几页
     */
    private void lockRecord(String deviceId, int page, Result result) throws NotInitialized {
        NetResponse netResponse = witsystmBleSDK.getDeviceManage().getDeviceService().lockRecord(deviceId, page);
        if (netResponse.getErr() == Error.SUCCESS.getCode()) {
            result.success(JSON.toJSONString(netResponse));
        } else {
            result.error(String.valueOf(netResponse.getErr()), netResponse.getMsg(), netResponse.getData());
        }
    }

    /**
     * 修改设备信息
     *
     * @param deviceId   设备ID
     * @param deviceName 名称
     */
    private void updateDeviceName(String deviceId, String deviceName, Result result) throws NotInitialized {
        NetResponse netResponse = witsystmBleSDK.getDeviceManage().getDeviceService().updateDeviceName(deviceId, deviceName);
        if (netResponse.getErr() == Error.SUCCESS.getCode()) {
            result.success(JSON.toJSONString(netResponse));
        } else {
            result.error(String.valueOf(netResponse.getErr()), netResponse.getMsg(), netResponse.getData());
        }
    }


    /**
     * 绑定网关
     *
     * @param superDeviceId 网关ID
     * @param deviceId      设备ID
     */
    private void bindRelay(String superDeviceId, String deviceId, Result result) throws NotInitialized {
        NetResponse netResponse = witsystmBleSDK.getDeviceManage().getDeviceService().bindRelay(superDeviceId, deviceId);

        if (netResponse.getErr() == Error.SUCCESS.getCode()) {
            result.success(JSON.toJSONString(netResponse));
        } else {
            result.error(String.valueOf(netResponse.getErr()), netResponse.getMsg(), netResponse.getData());
        }
    }


    /**
     * 解绑绑定网关
     *
     * @param deviceId 设备ID
     */
    private void unbindRelay(String deviceId, Result result) throws NotInitialized {
        NetResponse netResponse = witsystmBleSDK.getDeviceManage().getDeviceService().unbindRelay(deviceId);
        if (netResponse.getErr() == Error.SUCCESS.getCode()) {
            result.success(JSON.toJSONString(netResponse));
        } else {
            result.error(String.valueOf(netResponse.getErr()), netResponse.getMsg(), netResponse.getData());
        }
    }

}
