//
// Created by wits on 2022/5/20.
//

import Foundation
import Flutter

import WitsystemIosSDK

///蓝牙连接状态处理

public class BleConnectStateHandler: NSObject, FlutterStreamHandler {

    private var stateNumber: Int? = nil;
    private var bleDevice: BleDevice? = nil

    public func onListen(withArguments arguments: Any?, eventSink events: @escaping FlutterEventSink) -> FlutterError? {
        do {
            bleDevice = try WitsystemIosSDK.witsystemIosSDK.getBleDevice()
            if (stateNumber != nil) {
                bleDevice!.cancelBleDeviceConnectState(stateNumber)
            }


            stateNumber = bleDevice!.bleDeviceConnectState { deviceId, state in
                if (deviceId == nil) {

                    events(String(data: try! JSONEncoder().encode(ConnectState(deviceId: nil,
                            mac: nil,
                            newVal: state.rawValue,
                            oldVal: (state == BleConnectState.disconnected ? BleConnectState.connected : BleConnectState.disconnected).rawValue)), encoding: .utf8));
                    return
                }
                var mac = String(deviceId!.suffix(deviceId!.count - 5))
                mac.insert(":", at: mac.index(mac.startIndex, offsetBy: 10))
                mac.insert(":", at: mac.index(mac.startIndex, offsetBy: 8))
                mac.insert(":", at: mac.index(mac.startIndex, offsetBy: 6))
                mac.insert(":", at: mac.index(mac.startIndex, offsetBy: 4))
                mac.insert(":", at: mac.index(mac.startIndex, offsetBy: 2))

                events(String(data: try! JSONEncoder()
                        .encode(ConnectState(deviceId: deviceId,
                                mac: mac,
                                newVal: state.rawValue,
                                oldVal: (state == BleConnectState.disconnected ? BleConnectState.connected : BleConnectState.disconnected).rawValue)),
                        encoding: .utf8));
            }
        } catch let err {
            let notInitialized: NotInitialized = err as! NotInitialized
            events(FlutterError(code: notInitialized.code.description, message: notInitialized.error, details: nil));
        }
        return nil;
    }

    public func onCancel(withArguments arguments: Any?) -> FlutterError? {
        if (stateNumber != nil && bleDevice != nil) {
            bleDevice!.cancelBleDeviceConnectState(stateNumber)
        }
        stateNumber = nil;
        return nil;
    }


}

class ConnectState: Codable {

    var deviceId: String?

    var mac: String?

    var newVal: Int

    var oldVal: Int

    init(deviceId: String?, mac: String?, newVal: Int, oldVal: Int) {
        self.deviceId = deviceId;
        self.mac = mac;
        self.newVal = newVal;
        self.oldVal = oldVal;
    }


}
