///开锁对象
import 'package:flutter/services.dart';
import 'package:witsystem_device_plugin/vibrate/Vibrate.dart';
import 'dart:convert' as convert;
import '../WitsystemSDK.dart';

class Unlock extends WisystemSDk {
  factory Unlock() => _getInstance();

  static Unlock get instance => _getInstance();
  static Unlock? _instance;

  Unlock._internal();

  static Unlock _getInstance() {
    _instance ??= Unlock._internal();
    return _instance!;
  }

  ///开启附近设备
  bleUnlocks(
      {bool isVibrate = false, //开门成功是否震动
      Function(Map map)? success,
      Function(Map map)? foundMultipleDevices,
      Function(PlatformException e)? fail}) async {
    try {
      String json = await methodChannel.invokeMethod("bleUnlock");
      Map<String, dynamic> map = convert.json.decode(json);
      switch (map['type']) {
        case 'DEVICES':
          if (foundMultipleDevices != null) foundMultipleDevices(map);
          break;
        case 'UNLOCK':
          if (isVibrate) Vibrate().start();
          if (success != null) success(map);

          break;
      }
    } catch (e) {
      if (fail != null) fail(e as PlatformException);
    }
  }

  ///开启指定设备 deviceId代表开启指定设备
  bleUnlock(String deviceId, {bool isVibrate = false}) async {
    try {

      final map = {"deviceId": deviceId};
      var rtn = await methodChannel.invokeMethod("bleUnlock", map);
      if (isVibrate) Vibrate().start();
      return rtn;
    } catch (e) {
      rethrow;
    }
  }

  ///远程开启指定设备 deviceId代表开启指定设备
  longUnlock(String deviceId, {bool isVibrate = false}) async {
    try {
      final map = {"deviceId": deviceId};
      var rtn = await methodChannel.invokeMethod("longUnlock", map);
      if (isVibrate) Vibrate().start();
      return rtn;
    } catch (e) {
      rethrow;
    }
  }
}
